@@ -33,31 +33,35 @@
 #include <kkeybutton.h>
 #include <klistview.h>
 #include <klocale.h>
+#include <kmainwindow.h>
 #include <kmessagebox.h>
 #include <kpopupmenu.h>
 #include <kpushbutton.h>
 #include <kstandarddirs.h>
 #include <kdeversion.h>
 #include <kinputdialog.h>
+#include <ktoolbar.h>
 
 //app includes
 #include "resource.h"
-#include "quanta.h"
 #include "filecombo.h"
 
 #include "actionconfigdialog.h"
 #include "tagaction.h"
 #include "toolbartabwidget.h"
 
-ActionConfigDialog::ActionConfigDialog( QWidget* parent, const char* name, bool modal, WFlags fl, const QString& defaultAction )
+ActionConfigDialog::ActionConfigDialog(const QDict<ToolbarEntry> &toolbarList, KMainWindow* parent, const char* name, bool modal, WFlags fl, const QString& defaultAction )
     :ActionConfigDialogS( parent, name, modal, fl )
 {
+  m_mainWindow = parent;
+  m_toolbarList = toolbarList;
   buttonOk->setIconSet(SmallIconSet("button_ok"));
   buttonCancel->setIconSet(SmallIconSet("button_cancel"));
   buttonApply->setIconSet(SmallIconSet("apply"));
   //buttonHelp->setIconSet(SmallIconSet("help"));
 
   currentAction = 0L;
+  m_toolbarItem = 0L;
 
   connect(actionTreeView, SIGNAL(contextMenu(KListView *,QListViewItem *,const QPoint &)),
                           SLOT(slotContextMenu(KListView *,QListViewItem *,const QPoint &)));
@@ -67,6 +71,10 @@ ActionConfigDialog::ActionConfigDialog( QWidget* parent, const char* name, bool
                              SLOT(slotShortcutCaptured(const KShortcut &)));
   connect(scriptPath, SIGNAL(activated(const QString&)),
                       SLOT(slotTextChanged(const QString&)));
+  connect(this, SIGNAL(addToolbar()), m_mainWindow, SLOT(slotAddToolbar()));
+  connect(this, SIGNAL(removeToolbar(const QString&)), m_mainWindow, SLOT(slotRemoveToolbar(const QString&)));
+  connect(this, SIGNAL(deleteUserAction(KAction*)), m_mainWindow, SLOT(slotDeleteAction(KAction*)));
+  connect(this, SIGNAL(configureToolbars(const QString&)), m_mainWindow, SLOT(slotConfigureToolbars(const QString&)));
 //fill up the tree view with the toolbars and actions
   actionTreeView->setSorting(-1);
   allActionsItem = new KListViewItem(actionTreeView, i18n("All"));
@@ -74,16 +82,20 @@ ActionConfigDialog::ActionConfigDialog( QWidget* parent, const char* name, bool
   QListViewItem *item, *oldItem = allActionsItem;
   KAction *action;
   QString toolbarName;
+  QString toolbarId;
   ToolbarTabWidget *tb = ToolbarTabWidget::ref();
   QRegExp r("\\&(?!\\&)");
+  KActionCollection *ac = m_mainWindow->actionCollection();
   for (int i = 0; i < tb->count(); i++)
   {
     toolbarName = tb->label(i);
+    toolbarName.replace(r, "");
     item = new KListViewItem(actionTreeView, oldItem, i18n(toolbarName.utf8()));
     actionTreeView->insertItem(item);
 
+    toolbarId = tb->id(i);
     QListViewItem *oldActionItem = 0L;
-    ToolbarEntry *p_toolbar = quantaApp->toolbarList[toolbarName.lower()];
+    ToolbarEntry *p_toolbar = m_toolbarList[toolbarId];
     if (p_toolbar)
     {
       QDomNode node = p_toolbar->guiClient->domDocument().firstChild().firstChild().firstChild();
@@ -91,16 +103,16 @@ ActionConfigDialog::ActionConfigDialog( QWidget* parent, const char* name, bool
       {
         if (node.nodeName() == "Action")
         {
-          action = quantaApp->actionCollection()->action(node.toElement().attribute("name"));
+          action = ac->action(node.toElement().attribute("name"));
           if (action)
           {
             oldActionItem = new KListViewItem(item, oldActionItem, action->text().replace(r,""), action->shortcut().toString(), action->name());
-            oldActionItem->setPixmap(0, BarIcon(action->icon()) );
+            oldActionItem->setPixmap(0, SmallIcon(action->icon()) );
           }
         }
         node = node.nextSibling();
       }
-      if (tb->label(tb->currentPageIndex()) == toolbarName)
+      if (tb->label(tb->currentPageIndex()).replace(r, "") == toolbarName)
       {
          item->setOpen(true);
          if (item->firstChild())
@@ -126,11 +138,12 @@ ActionConfigDialog::ActionConfigDialog( QWidget* parent, const char* name, bool
     }
     oldItem = item;
   }
-  for (uint i = 0; i < quantaApp->actionCollection()->count(); i++)
+  uint acCount = ac->count();
+  for (uint i = 0; i < acCount; i++)
   {
-    action = quantaApp->actionCollection()->action(i);
+    action = ac->action(i);
     item = new KListViewItem(allActionsItem, action->text().replace(r, ""),  action->shortcut().toString(), action->name());
-    item->setPixmap(0, BarIcon(action->icon()) );
+    item->setPixmap(0, SmallIcon(action->icon()) );
   }
   allActionsItem->sortChildItems(0, true);
 
@@ -148,7 +161,7 @@ ActionConfigDialog::~ActionConfigDialog()
 
 void ActionConfigDialog::slotAddToolbar()
 {
-  quantaApp->slotAddToolbar();
+  emit addToolbar();
   QString toolbarName;
   QListViewItem *item;
   ToolbarTabWidget *tb = ToolbarTabWidget::ref();
@@ -177,20 +190,27 @@ void ActionConfigDialog::slotRemoveToolbar()
   }
   if (s != i18n("All"))
   {
-    if ( KMessageBox::warningYesNo(this, i18n("Do you really want to remove the \"%1\" toolbar?").arg(s)) == KMessageBox::Yes )
+      if ( KMessageBox::warningContinueCancel(this, i18n("Do you really want to remove the \"%1\" toolbar?").arg(s),QString::null,KStdGuiItem::del()) == KMessageBox::Continue )
     {
-      if (quantaApp->slotRemoveToolbar(s.lower()))
-      {
-        actionTreeView->setCurrentItem(allActionsItem);
-        delete item;
-      }
+      m_toolbarItem = item;
+      connect(m_mainWindow, SIGNAL(toolbarRemoved(const QString&)), SLOT(slotToolbarRemoved(const QString&)));
+      emit removeToolbar(s.lower());
     }
   }
 }
 
+void ActionConfigDialog::slotToolbarRemoved(const QString &/*name*/)
+{
+  actionTreeView->setCurrentItem(allActionsItem);
+  delete m_toolbarItem;
+  m_toolbarItem = 0L;
+  disconnect(m_mainWindow, SIGNAL(toolbarRemoved(const QString&)), this, SLOT(slotToolbarRemoved(const QString&)));
+}
+
 void ActionConfigDialog::slotEditToolbar()
 {
   QString toolbarName;
+  QString toolbarId;
   QListViewItem *oldItem;
   QListViewItem *item = actionTreeView->currentItem();
   if (item->parent())
@@ -198,15 +218,17 @@ void ActionConfigDialog::slotEditToolbar()
   toolbarName = item->text(0);
   if ( toolbarName != i18n("All"))
   {
-    quantaApp->slotConfigureToolbars(toolbarName +" <quanta>");
+    emit configureToolbars(toolbarName +" <quanta>");
 
     //update the tree view
     KAction *action;
+    KActionCollection *ac = m_mainWindow->actionCollection();
     ToolbarTabWidget *tb = ToolbarTabWidget::ref();
     for (int i = 0; i < tb->count(); i++)
     {
       toolbarName = tb->label(i);
-      ToolbarEntry *p_toolbar = quantaApp->toolbarList[toolbarName.lower()];
+      toolbarId = tb->id(i);
+      ToolbarEntry *p_toolbar = m_toolbarList[toolbarId];
       if (p_toolbar)
       {
         oldItem = actionTreeView->findItem(toolbarName, 0);
@@ -218,11 +240,11 @@ void ActionConfigDialog::slotEditToolbar()
         {
           if (node.nodeName() == "Action")
           {
-            action = quantaApp->actionCollection()->action(node.toElement().attribute("name"));
+            action = ac->action(node.toElement().attribute("name"));
             if (action)
             {
               oldItem = new KListViewItem(item, oldItem, action->text().replace(QRegExp("\\&(?!\\&)"),""), action->shortcut().toString(), action->name());
-              oldItem->setPixmap(0, BarIcon(action->icon()));
+              oldItem->setPixmap(0, SmallIcon(action->icon()));
             }
           }
           node = node.nextSibling();
@@ -244,7 +266,7 @@ void ActionConfigDialog::slotSelectionChanged(QListViewItem *item)
   if (currentAction && currentAction->inherits("TagAction"))
   {
     if ( buttonApply->isEnabled() &&
-         KMessageBox::questionYesNo(this, i18n("Do you want to save the changes made to this action?")) == KMessageBox::Yes )
+         KMessageBox::questionYesNo(this, i18n("Do you want to save the changes made to this action?"), QString::null, KStdGuiItem::save(), KStdGuiItem::discard()) == KMessageBox::Yes )
     {
       saveCurrentAction();
     }
@@ -254,10 +276,12 @@ void ActionConfigDialog::slotSelectionChanged(QListViewItem *item)
   if (item && item->depth() > 0)
   {
     TagAction *action = 0L;
+    KActionCollection *ac = m_mainWindow->actionCollection();
+    uint acCount = ac->count();
 //find the corresponding action
-    for (uint i = 0; i < quantaApp->actionCollection()->count(); i++)
+    for (uint i = 0; i < acCount; i++)
     {
-      KAction *a = quantaApp->actionCollection()->action(i);
+      KAction *a = ac->action(i);
       QString actionName = a->name();
       if (a && actionName == item->text(2) && a->inherits("TagAction"))
       {
@@ -309,7 +333,8 @@ void ActionConfigDialog::slotSelectionChanged(QListViewItem *item)
       for (int i = 0; i < tb->count(); i++)
       {
         QString toolbarName = tb->label(i);
-        ToolbarEntry *p_toolbar = quantaApp->toolbarList[toolbarName.lower()];
+        QString toolbarId = tb->id(i);
+        ToolbarEntry *p_toolbar = m_toolbarList[toolbarId];
         if (p_toolbar)
         {
           QDomNode node = p_toolbar->guiClient->domDocument().firstChild().firstChild().firstChild();
@@ -469,7 +494,7 @@ void ActionConfigDialog::saveCurrentAction()
     listItem = it.current();
     if (listItem->depth() > 0 && listItem->text(2) == currentAction->name())
     {
-      listItem->setPixmap(0, BarIcon(actionIcon->icon()));
+      listItem->setPixmap(0, SmallIcon(actionIcon->icon()));
       listItem->setText(0, lineText->text());
       listItem->setText(1, currentAction->shortcut().toString());
     }
@@ -583,7 +608,8 @@ void ActionConfigDialog::saveCurrentAction()
   for (int i = 0; i < tb->count(); i++)
   {
     QString toolbarName = tb->label(i);
-    ToolbarEntry *p_toolbar = quantaApp->toolbarList[toolbarName.lower()];
+    QString toolbarId = tb->id(i);
+    ToolbarEntry *p_toolbar = m_toolbarList[toolbarId];
     bool isOnToolbar = false;
     if (p_toolbar)
     {
@@ -644,7 +670,7 @@ void ActionConfigDialog::saveCurrentAction()
           if (placeOnToolbar)
           {
               listItem = new KListViewItem(listItem, after, lineText->text(), currentAction->shortcut().toString(), currentAction->name());
-              listItem->setPixmap(0, BarIcon(actionIcon->icon()));
+              listItem->setPixmap(0, SmallIcon(actionIcon->icon()));
           }
         }
       }
@@ -694,7 +720,7 @@ void ActionConfigDialog::slotShortcutCaptured(const KShortcut &shortcut)
 
   if (global.isEmpty())
   {
-    QPtrList<KXMLGUIClient> clients = quantaApp->guiFactory()->clients();
+    QPtrList<KXMLGUIClient> clients = m_mainWindow->guiFactory()->clients();
     for( QPtrListIterator<KXMLGUIClient> it( clients ); it.current(); ++it )
     {
         KActionCollection *ac = (*it)->actionCollection();
@@ -743,7 +769,7 @@ void ActionConfigDialog::accept()
 
 void ActionConfigDialog::reject()
 {
-  if (buttonApply->isEnabled() && KMessageBox::questionYesNo(this, i18n("Do you want to save the changes made to this action?")) == KMessageBox::Yes  )
+  if (buttonApply->isEnabled() && KMessageBox::questionYesNo(this, i18n("Do you want to save the changes made to this action?"), QString::null, KStdGuiItem::save(), KStdGuiItem::discard()) == KMessageBox::Yes  )
       saveCurrentAction();
   ActionConfigDialogS::reject();
 }
@@ -755,10 +781,10 @@ void ActionConfigDialog::slotNewAction()
   el.setAttribute( "name", "user_"+KApplication::randomString(10) );
   el.setAttribute( "icon", "ball" );
 
-  currentAction = new TagAction(&el, quantaApp->actionCollection());
+  currentAction = new TagAction(&el, m_mainWindow->actionCollection());
 #if KDE_IS_VERSION(3,2,90)
     //add the actions to every toolbar xmlguiclient
-    QDictIterator<ToolbarEntry> it(quantaApp->toolbarList);
+    QDictIterator<ToolbarEntry> it(m_toolbarList);
     while (it.current())
     {
       it.current()->guiClient->actionCollection()->insert(currentAction);
@@ -770,11 +796,11 @@ void ActionConfigDialog::slotNewAction()
   static_cast<TagAction*>(currentAction)->setModified(true);
   QListViewItem *currentItem = actionTreeView->currentItem();
   QListViewItem *item = new KListViewItem(allActionsItem);
-  QString actionText = QString("Action_%1").arg(quantaApp->actionCollection()->count());
+  QString actionText = QString("Action_%1").arg(m_mainWindow->actionCollection()->count());
   currentAction->setText(actionText);
   item->setText(2, currentAction->name());
   item->setText(0, actionText);
-  item->setPixmap(0, BarIcon("ball"));
+  item->setPixmap(0, SmallIcon("ball"));
   allActionsItem->sortChildItems(0, true);
   if (currentItem->parent() && currentItem->parent() == allActionsItem)
   {
@@ -788,7 +814,7 @@ void ActionConfigDialog::slotNewAction()
     item = new KListViewItem(parentItem, currentItem);
     item->setText(0, actionText);
     item->setText(2, currentAction->name());
-    item->setPixmap(0, BarIcon("ball"));
+    item->setPixmap(0, SmallIcon("ball"));
     actionTreeView->setCurrentItem(item);
     if (parentItem != allActionsItem)
     {
@@ -803,10 +829,10 @@ void ActionConfigDialog::slotNewAction()
 
 void ActionConfigDialog::slotDeleteAction()
 {
-  if ( KMessageBox::warningYesNo(this, i18n("<qt>Removing the action removes all the references to it.\nAre you sure you want to remove the <b>%1</b> action?</qt>").arg(currentAction->text())) == KMessageBox::Yes )
+    if ( KMessageBox::warningContinueCancel(this, i18n("<qt>Removing the action removes all the references to it.\nAre you sure you want to remove the <b>%1</b> action?</qt>").arg(currentAction->text()),QString::null,KStdGuiItem::del()) == KMessageBox::Continue )
   {
     QString actionName = currentAction->name();
-    quantaApp->slotDeleteAction(currentAction);
+    emit deleteUserAction(currentAction);
     currentAction = 0L;
     //update the tree view
     QListViewItem *listItem;
@@ -847,10 +873,10 @@ void ActionConfigDialog::slotAddContainerToolbar()
 
   if (lst.count() > 0)
   {
-    bool ok = FALSE;
+    bool ok = false;
     QString res = KInputDialog::getItem(
                     i18n( "Add Action to Toolbar" ),
-                    i18n( "Please select a toolbar:" ), lst, 0, FALSE, &ok, this );
+                    i18n( "Please select a toolbar:" ), lst, 0, false, &ok, this );
 
     if (ok)
     {
