@@ -499,16 +499,24 @@ bool KafkaDocument::buildKafkaNodeFromNode(Node *node, bool insertNode)
 	Node *n, *parent;
 	int i;
 
-//     This is a hack to not created DOM::Nodes from quanta empty nodes if outside body, because KHTML
-//     moves a node in that condition into the body and then the trees become desynchronized.
-    bool isInsideBody = false;
-    if(!m_currentDoc->defaultDTD()->name.contains("HTML", false))
-        isInsideBody = true;
-    else
-        isInsideBody = kafkaCommon::hasParent(node, "body");
+//     Don't create DOM::Nodes from Quanta empty nodes outside the body or inside other not allowed element, or KHTML
+//     will give us problems.
+    bool canInsertEmptyNode = false;
+    if(node->tag->type == Tag::Empty)
+    {
+        if(!m_currentDoc->defaultDTD()->name.contains("HTML", false))
+            canInsertEmptyNode = true;
+        else
+            canInsertEmptyNode = kafkaCommon::hasParent(node, "body");
+        
+        Node* parent_node = node->parent;
+        QTag* parent_node_description_tag = QuantaCommon::tagFromDTD(parent_node);
+        if(parent_node_description_tag && !parent_node_description_tag->isChild(node, false, true))
+            canInsertEmptyNode = false;
+    }
     
     if(node->tag->type == Tag::XmlTag || 
-       ((node->tag->type == Tag::Text || (node->tag->type == Tag::Empty && isInsideBody)) && !node->insideSpecial))
+       ((node->tag->type == Tag::Text || (node->tag->type == Tag::Empty && canInsertEmptyNode)) && !node->insideSpecial))
     {
 		str = node->tag->name.lower();
 
@@ -709,29 +717,36 @@ bool KafkaDocument::buildKafkaNodeFromNode(Node *node, bool insertNode)
 			node->setLeafNode(ptDomNode);
 			mainEnhancer->enhanceNode(node, parentNode, nextNode);
 
-			if(nextNode.isNull())
-			{
-				if(!kafkaCommon::insertDomNode(newNode, parentNode))
-				{
-					disconnectDomNodeFromQuantaNode(newNode);
-					if(node->rootNode())
-						delete node->rootNode();
-					node->setRootNode(0L);
-					return false;
-				}
-			}
-			else
-			{
-				if(!kafkaCommon::insertDomNode(newNode, parentNode, nextNode))
-				{
-					disconnectDomNodeFromQuantaNode(newNode);
-					if(node->rootNode())
-						delete node->rootNode();
-					node->setRootNode(0L);
-					return false;
-				}
-			}
-		}
+            QTag* qTag = QuantaCommon::tagFromDTD(getCurrentDoc()->defaultDTD(),
+                    parentNode.nodeName().string());
+
+            if(qTag->isChild(node, false))
+            {
+                
+                if(nextNode.isNull())
+                {
+                    if(!kafkaCommon::insertDomNode(newNode, parentNode))
+                    {
+                        disconnectDomNodeFromQuantaNode(newNode);
+                        if(node->rootNode())
+                            delete node->rootNode();
+                        node->setRootNode(0L);
+                        return false;
+                    }
+                }
+                else
+                {
+                    if(!kafkaCommon::insertDomNode(newNode, parentNode, nextNode))
+                    {
+                        disconnectDomNodeFromQuantaNode(newNode);
+                        if(node->rootNode())
+                            delete node->rootNode();
+                        node->setRootNode(0L);
+                        return false;
+                    }
+                }
+            }
+        }
 		else
 		{
 			ptDomNode = new DOM::Node(newNode);
@@ -852,7 +867,9 @@ QString KafkaDocument::getDecodedChar(const QString &encodedChar)
 QString KafkaDocument::getDecodedText(const QString &a_encodedText, bool translateWhiteSpacesAndLineBreaks,
 		bool removeLeftWhitespaces, bool removeRightWhitespaces)
 {
-        QString encodedText = a_encodedText;
+  QString encodedText = a_encodedText;
+  if (encodedText.isEmpty())
+    encodedText = " ";
 	QString decodedChar;
 	int i, j;
 #ifdef LIGHT_DEBUG
