@@ -49,6 +49,7 @@
 #include <kpassdlg.h>
 #include <kpushbutton.h>
 #include <ktempfile.h>
+#include <kuser.h>
 
 //standard includes
 #include <time.h>
@@ -161,11 +162,25 @@ void ProjectUpload::slotBuildTree()
  totalProgress->setValue(0);
  totalText->setText(i18n("Scanning project files..."));
 
- QDict<KFileItem> projectDirFiles = QExtFileInfo::allFilesDetailed(m_project->projectBaseURL(), "*");
-  QTime t;
-  t.start();
-
  KURL u = m_project->projectBaseURL();
+ if (!startUrl.isEmpty()) 
+ {
+   u = QExtFileInfo::toAbsolute(startUrl, u);
+ }
+ QDict<KFileItem> projectDirFiles;
+
+ if (startUrl.isEmpty() || strUrl.endsWith("/")) //upload a folder 
+ {
+   projectDirFiles = QExtFileInfo::allFilesDetailed(u, "*");
+ } else
+ {
+   projectDirFiles.insert(u.url(), new KFileItem(KFileItem::Unknown, KFileItem::Unknown, u, true));
+ }
+
+ QTime t;
+ t.start();
+
+ u = m_project->projectBaseURL();
  KURL absUrl = u;
  for (uint i = 0; i < nl.count(); i++)
  {
@@ -675,6 +690,12 @@ void ProjectUpload::fillProfileDlg(UploadProfileDlgS *profileDlg)
 
 void ProjectUpload::readProfileDlg(UploadProfileDlgS *profileDlg)
 {
+  QString path = profileDlg->linePath->text();
+  if (path.startsWith("~/"))
+  {
+    KUser user;
+    path = user.homeDir() + path.mid(1);
+  }
   m_currentProfileElement.setAttribute("name", profileDlg->lineProfileName->text());
   m_currentProfileElement.setAttribute("remote_host", profileDlg->lineHost->text());
   m_currentProfileElement.setAttribute("user", profileDlg->lineUser->text());
