@@ -19,7 +19,7 @@
 #define QUANTA_H
 
 #define QUANTA_PACKAGE "quanta"
-#define QUANTA_VERSION "3.3.2"
+#define QUANTA_VERSION "3.4.3"
 
 #define IDS_STATUS      1
 #define IDS_INS_OVR     2
@@ -57,6 +57,7 @@ class QNewDTEPStuff;
 class QNewToolbarStuff;
 class QNewTemplateStuff;
 class QNewScriptStuff;
+class QNewDocStuff;
 
 class QTabWidget;
 class QWidgetStack;
@@ -96,20 +97,10 @@ namespace KTextEditor
   class View;
 }
 
-class SpellChecker;
 struct DirInfo;
 struct EventAction;
+struct ToolbarEntry;
 
-typedef struct ToolbarEntry{
-  KXMLGUIClient *guiClient;
-  QDomDocument *dom;
-  QPopupMenu *menu;
-  KURL url;
-  QString name;
-  bool user;
-  bool visible;
-  bool nameModified; ///< True if the name of the toolbar was modified by Quanta on loading:" (n)" was added
-};
 
 /**
   * The base class for Quanta application windows.
@@ -154,12 +145,14 @@ public:
   QStringList openedURLs() const;
   QString saveCurrentFile();
   QStringList tagAreas(const QString& name, bool includeCoordinates, bool skipFoundContent) const;
-  QString documentFolderForURL(QString url);
-  QString urlWithPreviewPrefix(QString url);
+  QString documentFolderForURL(const QString &url);
+  QString urlWithPreviewPrefix(const QString &url);
+  void addFileToProject(const QString &url);
+  void addFolderToProject(const QString &url);
   /** Capture DCOP signals from KXsldbgPart or similar plugin */
-  void newCursorPosition(QString file, int lineNumber, int columnNumber);
-  void newDebuggerPosition(QString file, int lineNumber);
-  void openFile(QString file, int lineNumber, int columnNumber);
+  void newCursorPosition(const QString &file, int lineNumber, int columnNumber);
+  void newDebuggerPosition(const QString &file, int lineNumber);
+  void openFile(const QString &file, int lineNumber, int columnNumber);
 
   /** reparse current document and initialize node. */
   void reparse(bool force);
@@ -168,8 +161,6 @@ public:
 
   //return the old Cursor position
   void oldCursorPos(uint &line, uint &col) {line = oldCursorLine; col = oldCursorCol;}
-  /** Loads the toolbars for dtd named dtdName and unload the ones belonging to oldDtdName. */
-  void loadToolbarForDTD(const QString& dtdName);
 
   QStringList selectors(const QString& tag);
   QStringList idSelectors();
@@ -177,8 +168,8 @@ public:
   /** Show the toolbar which is in url. If it was not loaded yet, it loads the
       toolbar from the file */
   void showToolbarFile(const KURL &url);
-  /** Restores the original document from the temporary backup */
-  void restoreFromTempfile(Document *w);
+
+  void setParserEnabled(bool enabled) {m_parserEnabled = enabled;}
 
     /** tabs for left panel */
   DocTreeView *dTab;
@@ -194,7 +185,6 @@ public:
   /** Returns the baseURL of the document. */
   KURL baseURL();
 
-  bool enableIdleTimer(bool enable);
   void startIdleTimer();
 
   /** Called when a document was closed. Resets some variables. */
@@ -215,15 +205,18 @@ signals: // Signals
 
   /** Emitted when some kind of event that can have associated actions has happened. */
   void eventHappened(const QString&, const QString&, const QString& );
+  
+  void toolbarRemoved(const QString&);
 
 public slots:
   void slotFileNew();
   void slotFileOpen();
-  void slotFileOpen(const KURL &);
-  void slotFileOpen( const KURL &, const QString& );
-  void slotFileOpen( const KURL::List &urls, const QString& encoding );
+  void slotFileOpen(const KURL &url);
+  void slotFileOpen(const KURL &url, const QString &encoding);
+  void slotFileOpen(const KURL &url, const QString &encoding, bool readOnly);
+  void slotFileOpen(const KURL::List &urls, const QString& encoding);
   void slotFileSave();
-  bool slotFileSaveAs();
+  bool slotFileSaveAs(QuantaView *viewToSave = 0L);
   void slotFileSaveAsLocalTemplate();
   void slotFileSaveAsProjectTemplate();
   void slotFileSaveSelectionAsLocalTemplate();
@@ -237,9 +230,9 @@ public slots:
   void slotFileCloseAll();
   void slotFileQuit();
 
-  void slotOpenFileInPreview(const KURL&);
-  void slotImageOpen(const KURL&);
-  void slotFileOpenRecent(const KURL&);
+  void slotOpenFileInPreview(const KURL &url);
+  void slotImageOpen(const KURL &url);
+  void slotFileOpenRecent(const KURL &url);
   void slotInsertTag(const KURL&, DirInfo);
 
   void slotEditFindInFiles();
@@ -298,10 +291,8 @@ public slots:
 
   void selectArea(int line1, int col1, int line2, int col2);
 
-  /** No descriptions */
-  void slotInsertFile(const KURL&);
-  /** No descriptions */
-  void slotSyntaxCheckDone();
+  /** Insert the content of the url into the current document. */
+  void slotInsertFile(const KURL& url);
   /** Sends a toolbar in mail. */
   void slotSendToolbar();
   /** Removes a user toolbar from the toolbars. */
@@ -312,6 +303,8 @@ public slots:
   void slotSaveLocalToolbar();
   /** Saves a toolbar as project specific. */
   void slotSaveProjectToolbar();
+  /** Loads the toolbars for dtd named dtdName and unload the ones belonging to oldDtdName. */
+  void slotLoadToolbarForDTD(const QString& dtdName);
   /** Load an user toolbar from the disk. */
   void slotLoadToolbarFile(const KURL& url);
   /** Load an user toolbar from the disk. */
@@ -330,19 +323,25 @@ public slots:
   void slotRemoveAction(const QString&, const QString& actionName);
   /** Edit the action */
   void slotEditAction(const QString&);
+  /** Creates a new, empty action */
+  void slotNewAction();
   /** Creates a script action for a_scriptURL using the a_interpreter as the script
   interpreter application */
   void slotAssignActionToScript(const KURL&a_scriptURL, const QString& a_interpreter);
 
-  /** No descriptions */
+  /** Change the DTD/DTEP of the current document. */
   void slotChangeDTD();
-  /** No descriptions */
+  /** Edit a DTD/DTEP description. */
+  void slotEditDTD();
+  /** Called when the user explicetly requested the completion box.*/
   void slotShowCompletion();
-  /** No descriptions */
+  /** Called when the user explicetly requested the completion hint.*/
   void slotShowCompletionHint();
-  /** No descriptions */
-  void slotParsingDTDChanged(const QString& newDTDName);
-  /** No descriptions */
+  /** Show or hide the groups for dtepName in the structure tree.
+    The special value of "clear" for dtepName means show groups
+    for all DTEPs found in the document.*/
+  void slotShowGroupsForDTEP(const QString& dtepName, bool show);
+  /** Build the project specific toolbar menu. */
   void slotBuildPrjToolbarsMenu();
 
   void slotReparse();
@@ -350,6 +349,8 @@ public slots:
 
   void slotExpandAbbreviation();
 
+  /** Format the code of the document */
+  void slotCodeFormatting();
   /** Show the Document Properties Dialog */
   void slotDocumentProperties();
   /** Show the Document Properties Dialog
@@ -379,6 +380,8 @@ public slots:
 
   /** Called when the preview or documentation part is deleted. */
   void slotHTMLPartDeleted(QObject *object);
+  
+  bool slotEnableIdleTimer(bool enable);
 
 //Overridden KMdiMainFrm slots
   virtual void closeAllViews();
@@ -390,6 +393,9 @@ public slots:
   /** appends all visible user toolbar urls to the list */
   void slotGetUserToolbarFiles(KURL::List *list);
 
+  // Get saved position of dock
+  KDockWidget::DockPosition prevDockPosition(QWidget* dock, KDockWidget::DockPosition def);
+  
 protected slots:
   void slotDockWidgetHasUndocked(KDockWidget *widget);
   void slotPreviewBeingClosed();
@@ -409,12 +415,20 @@ protected slots:
   void slotUploadDTEP();
   /** Downloads a toolbar from the main server */
   void slotDownloadToolbar();
+  /** Uploads a toolbar to the main server */
+  void slotUploadToolbar();
   /** Enable/Disable Smart Tag Insertion */
   void slotSmartTagInsertion();
   /** Downloads a template from the main server */
   void slotDownloadTemplate();
+  /** Uploads a template to the main server */
+  void slotUploadTemplate(const QString &fileName);
   /** Downloads a script from the main server */
   void slotDownloadScript();
+  /** Uploads a script to the main server */
+  void slotUploadScript(const QString &fileName);
+  /** Downloads a documentation from the main server */
+  void slotDownloadDoc();
   /** Shows tip of the day */
   void slotHelpTip();
   /** Show the user mailing list sign up */
@@ -465,15 +479,24 @@ protected slots:
   void slotCopy();
   void slotPaste();
 
-//Tools
-  void slotSpellcheck ();
-
   void slotShowSourceEditor();
   void slotShowVPLAndSourceEditor();
   void slotShowVPLOnly();
   void slotTabDragged(QWidget *widget);
+  void slotTabMoved(int from, int to);
+  void slotTabAboutToMove(int from, int to);
 
 protected:
+  /** Create a DTEP tarball which can be uploaded or sent in email. Returns
+  *   the name of the created file or QString::null if creation has failed.
+  */
+  QString createDTEPTarball();
+  
+  /** Create a toolbar tarball which can be uploaded or sent in email. Returns
+  *   the name of the created file or QString::null if creation has failed.
+  */
+  QString createToolbarTarball();
+  
   /** Ask for save all the modified user toolbars. */
   bool removeToolbars();
   /** Returns true if all toolbars are hidden, false otherwise. */
@@ -528,15 +551,13 @@ private:
   // DOC & VIEW
   QuantaDoc  *m_doc;
 
-  bool exitingFlag;
-
   /** parsered tree of document  */
   QTimer *statusbarTimer;
 
   // ACTIONS
   KRecentFilesAction *projectToolbarFiles;
 
-  KToggleAction *showKafkaAction, *showDTDToolbar;
+  KToggleAction *showVPLAction, *showSourceAction, *showVPLSourceAction, *showDTDToolbar;
   KToolBarPopupAction *showPreviewAction;
 
   KAction *saveAction, *saveAllAction;
@@ -558,8 +579,8 @@ private:
   uint oldCursorCol;
   bool m_previewVisible;
   bool m_noFramesPreview;
+  bool m_parserEnabled; ///< enables/disables reparsing. If false, even a forced reparse is ignored (used when opening multiple files)
 
-  KProcess* m_execCommandPS;
   QString m_scriptOutput;
 
 protected: // Protected attributes
@@ -582,14 +603,14 @@ protected: // Protected attributes
   DCOPQuanta *dcopQuanta;
   KParts::PartManager *m_partManager;  ///< the pointer to the part manager
   QGuardedPtr<KTextEditor::View> m_oldKTextEditor;  ///< remembers the last activated GUI
-  int m_cvsMenuId;
   QNewDTEPStuff *m_newDTEPStuff;
   QNewToolbarStuff *m_newToolbarStuff;
   QNewTemplateStuff *m_newTemplateStuff;
   QNewScriptStuff *m_newScriptStuff;
+  QNewDocStuff *m_newDocStuff;
+  QDict<ToolbarEntry> m_toolbarList;
 
 public: //TODO: check if it's worth to make a read method for them
-  QDict<ToolbarEntry> toolbarList;
   KRecentFilesAction *fileRecent;
   /** True when the whole quanta is initialized. */
   bool quantaStarted;
