@@ -319,10 +319,11 @@ void QuantaInit::initStatusBar()
 
   progressBar = new KProgress(m_quanta->statusBar());
   progressBar->setTextEnabled(false);
+  progressBar->setMaximumHeight(progressBar->fontMetrics().height());
   progressBar->show();
   m_quanta->statusBar()->insertItem(i18n(IDS_DEFAULT),IDS_STATUS, 1);
   m_quanta->statusBar()->addWidget(progressBar);
-  m_quanta->statusBar()->insertFixedItem(" XXX ",     IDS_INS_OVR  );
+  m_quanta->statusBar()->insertItem("",               IDS_INS_OVR  );
   m_quanta->statusBar()->insertFixedItem(" * ",       IDS_MODIFIED );
   m_quanta->statusBar()->insertFixedItem(i18n("Line: 00000 Col: 000"), IDS_STATUS_CLM, true);
 
@@ -439,9 +440,11 @@ void QuantaInit::initView()
    m_kafkaDocument->getKafkaWidget()->view()->setMinimumHeight(50);
    m_kafkaDocument->readConfig(quantaApp->config());
    loadVPLConfig();
-  (void) ToolbarTabWidget::ref(quantaApp);
+  ToolbarTabWidget *toolBarTab = ToolbarTabWidget::ref(quantaApp);
   //set the toolview and close button style before the GUI is created
-  m_config->setGroup  ("General Options");
+  m_config->setGroup("General Options");
+  int iconTextMode = KToolBar::iconTextSetting();
+  toolBarTab->setIconText(KToolBar::IconText(iconTextMode));
   qConfig.toolviewTabs = m_config->readNumEntry("MDI style", KMdi::IconOnly);
   m_quanta->initTabWidget();
 
@@ -548,7 +551,7 @@ void QuantaInit::readOptions()
   qConfig.updateClosingTags = m_config->readBoolEntry("Update Closing Tags", true);
   qConfig.replaceAccented = m_config->readBoolEntry("Replace Accented Chars", false);
 
-  qConfig.defaultEncoding = m_config->readEntry("Default encoding", QTextCodec::codecForLocale()->name());
+  qConfig.defaultEncoding = m_config->readEntry("Default encoding", "UTF8");
 
   StructTreeView::ref()->setFollowCursor( m_config->readBoolEntry("Follow Cursor", true));
 
@@ -623,17 +626,23 @@ void QuantaInit::openLastFiles()
   m_config->setGroup("General Options");
 
   QStringList urls = m_config->readPathListEntry("List of opened files");
+  QStringList encodings = m_config->readPathListEntry("Encoding of opened files");
   m_quanta->m_doc->blockSignals(true);
+  m_quanta->setParserEnabled(false);
+  uint i = 0;
   for ( QStringList::Iterator it = urls.begin(); it != urls.end(); ++it )
   {
     KURL fu;
     QuantaCommon::setUrl(fu, *it);
 
     if (!ViewManager::ref()->isOpened(fu) && (!isPrj || fu.isLocalFile()))
-      m_quanta->slotFileOpen(fu);
+      m_quanta->slotFileOpen(fu, encodings[i]);
+    i++;
   }
   m_config->sync();
   m_quanta->m_doc->blockSignals(false);
+  m_quanta->setParserEnabled(true);
+  m_quanta->reparse(true);
   Document *w = ViewManager::ref()->activeDocument();
   if (w) //w==0 might happen on quick close on startup
   {
@@ -1223,7 +1232,8 @@ void QuantaInit::recoverCrashed(QStringList& recoveredFileNameList)
 
 QString QuantaInit::searchPathListEntry(const QString& backedUpUrl,const QString& autosavedUrls)
 {
-  KURL k(backedUpUrl);
+  KURL k;
+  QuantaCommon::setUrl(k, backedUpUrl);
   QStringList autosavedUrlsList = QStringList::split(",", autosavedUrls);
   QStringList::Iterator autosavedUrlsIt;
   for (autosavedUrlsIt = autosavedUrlsList.begin();
