@@ -1,9 +1,9 @@
 /***************************************************************************
                           quantaview.cpp  -  description
                              -------------------
-    begin                : ï¿½ï¿½ï¿½ï¿½ 9 13:29:57 EEST 2000
+    begin                : Ã¯Â¿Â½Ã¯Â¿Â½Ã¯Â¿Â½Ã¯Â¿Â½ 9 13:29:57 EEST 2000
     copyright            : (C) 2000 by Dmitry Poplavsky & Alexander Yakovlev & Eric Laffoon <pdima@users.sourceforge.net,yshurik@linuxfan.com,sequitur@easystreet.com>
-                           (C) 2001-2004 Andras Mantia <amantia@kde.org>
+                           (C) 2001-2005 Andras Mantia <amantia@kde.org>
  ***************************************************************************/
 
 /***************************************************************************
@@ -35,6 +35,7 @@
 #include <kaction.h>
 #include <kdebug.h>
 #include <kdirwatch.h>
+#include <khtmlview.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
 #include <kmenubar.h>
@@ -45,12 +46,11 @@
 #include <kparts/partmanager.h>
 #include <kstatusbar.h>
 
-#ifdef BUILD_KAFKAPART
 #include "undoredo.h"
 #include "wkafkapart.h"
-#endif
 
 #include <ktexteditor/document.h>
+#include <ktexteditor/selectioninterface.h>
 #include <ktexteditor/view.h>
 #include <ktexteditor/viewcursorinterface.h>
 
@@ -75,20 +75,18 @@
 
 #include "tagdialog.h"
 
-QuantaView::QuantaView(QWidget *parent, const char *name )
+QuantaView::QuantaView(QWidget *parent, const char *name, const QString &caption )
   : KMdiChildView(parent, name)
   , m_document(0L)
   , m_plugin(0L)
   , m_customWidget(0L)
-#ifdef BUILD_KAFKAPART
+  , m_kafkaDocument(0L)
   , m_currentFocus(SourceFocus)
-#endif
 {
-#ifdef BUILD_KAFKAPART
+  setMDICaption(caption);
 //Connect the VPL update timers
   connect(&m_sourceUpdateTimer, SIGNAL(timeout()), this, SLOT(sourceUpdateTimerTimeout()));
   connect(&m_VPLUpdateTimer, SIGNAL(timeout()), this, SLOT(VPLUpdateTimerTimeout()));
-#endif
 
 //create the source and VPL holding widgets
   m_documentArea = new QWidget(this);
@@ -129,46 +127,47 @@ QuantaView::~QuantaView()
 
 bool QuantaView::mayRemove()
 {
-    emit hidePreview();
-    if (m_plugin)
-   {
-       m_plugin->unload(false);
-   } else
-   {
-     if (m_customWidget)
-         m_customWidget->reparent(0L, 0, QPoint(), false);
-     if (!saveModified())
+  emit hidePreview();
+  if (m_plugin)
+  {
+    m_plugin->unload(false);
+  } else
+  {
+    if (m_customWidget)
+        m_customWidget->reparent(0L, 0, QPoint(), false);
+    if (!saveModified())
           return false;
-     if (dynamic_cast<QuantaView *>(quantaApp->activeWindow()) == this)
-     {
-         parser->setSAParserEnabled(false);
-         delete baseNode;
-         baseNode = 0L;
-     }
-      if (m_document)
+    if (static_cast<QuantaView *>(quantaApp->activeWindow()) == this)
+    {
+        parser->setSAParserEnabled(false);
+        delete baseNode;
+        baseNode = 0L;
+        if (m_kafkaDocument && m_kafkaDocument->getKafkaWidget() && m_kafkaDocument->getKafkaWidget()->view())
+          m_kafkaDocument->getKafkaWidget()->view()->reparent(0, 0, QPoint(), false);
+    }
+    if (m_document)
+    {
+      KURL url = m_document->url();
+      Project::ref()->saveBookmarks(url, dynamic_cast<KTextEditor::MarkInterface*>(m_document->doc()));
+      emit eventHappened("before_close", url.url(), QString::null);
+      m_currentViewsLayout = -1;
+      //slotSetSourceLayout(); //set the layout to source only, otherwise it crashes...
+     // m_kafkaDocument->getKafkaWidget()->view()->reparent(0, 0, QPoint(), false);
+//      m_document->closeTempFile();
+      if (!m_document->isUntitled() && url.isLocalFile())
       {
-        KURL url = m_document->url();
-        emit eventHappened("before_close", url.url(), QString::null);
-        m_currentViewsLayout  = -1;
-#ifdef BUILD_KAFKAPART
-        slotSetSourceLayout(); //set the layout to source only, otherwise it crashes...
-        m_kafkaDocument->getKafkaWidget()->view()->reparent(0, 0, QPoint(), false);
-#endif
-          m_document->closeTempFile();
-          if (!m_document->isUntitled() && url.isLocalFile())
-          {
-            fileWatcher->removeFile(url.path());
-            kdDebug(24000) << "removeFile[mayRemove]: " << url.path() << endl;
-          }
-          quantaApp->menuBar()->activateItemAt(-1);
-          quantaApp->guiFactory()->removeClient(m_document->view());
-          emit eventHappened("after_close", url.url(), QString::null);
+        fileWatcher->removeFile(url.path());
+//        kdDebug(24000) << "removeFile[mayRemove]: " << url.path() << endl;
       }
+      quantaApp->menuBar()->activateItemAt(-1);
+      quantaApp->guiFactory()->removeClient(m_document->view());
+      emit eventHappened("after_close", url.url(), QString::null);
+    }
 /*      kdDebug(24000) << "Calling reparse from close " << endl;
       parser->setSAParserEnabled(true);
       quantaApp->reparse(true);*/
-   }
-   return true;
+  }
+  return true;
 }
 
 void QuantaView::addDocument(Document *document)
@@ -176,13 +175,11 @@ void QuantaView::addDocument(Document *document)
    if (!document)
      return;
    m_document = document;
-   connect(m_document, SIGNAL(editorGotFocus()),
-                  this, SLOT(slotSourceGetFocus()));
-    connect(m_document->view(), SIGNAL(cursorPositionChanged()), this, SIGNAL(cursorPositionChanged()));
+   connect(m_document, SIGNAL(editorGotFocus()), this, SLOT(slotSourceGetFocus()));
+   connect(m_document->view(), SIGNAL(cursorPositionChanged()), this, SIGNAL(cursorPositionChanged()));
 
 
-#ifdef BUILD_KAFKAPART
-   m_kafkaDocument =KafkaDocument::ref();
+   m_kafkaDocument = KafkaDocument::ref();
 
   connect(m_kafkaDocument->getKafkaWidget(), SIGNAL(hasFocus(bool)),
     this, SLOT(slotVPLGetFocus(bool)));
@@ -196,13 +193,10 @@ void QuantaView::addDocument(Document *document)
    m_curCol = m_curLine = m_curOffset = 0;
 
 //init the VPL part
-#ifdef BUILD_KAFKAPART
   m_currentViewsLayout = SourceOnly;//to correctly reload the timers.
 
   reloadUpdateTimers();
-#endif
 
-#endif
 
    m_currentViewsLayout = -1; //force loading of this layout
    int currentViewsLayout = SourceOnly;
@@ -222,7 +216,7 @@ void QuantaView::addDocument(Document *document)
 
 void QuantaView::addPlugin(QuantaPlugin *plugin)
 {
-   ToolbarTabWidget *m_toolbarTab = ToolbarTabWidget::ref(this);
+   ToolbarTabWidget *m_toolbarTab = ToolbarTabWidget::ref();
    m_toolbarTab->reparent(0, 0, QPoint(), false);
    m_plugin = plugin;
    m_splitter->hide();
@@ -357,16 +351,13 @@ void QuantaView::slotSetSourceLayout()
    if (m_currentViewsLayout == SourceOnly || !m_document)
      return;
 
-#ifdef BUILD_KAFKAPART
    if(m_currentViewsLayout == SourceAndVPL)
      m_splitterSizes = m_splitter->sizes();
-#endif
 
    KToggleAction *ta = (KToggleAction *) quantaApp->actionCollection()->action( "show_quanta_editor" );
    if (ta)
       ta->setChecked(true);
 
-#ifdef BUILD_KAFKAPART
    //hide the VPL widget, reload the source if necessary
    if ((m_currentViewsLayout == SourceAndVPL && m_kafkaDocument->getKafkaWidget()->view()->hasFocus()) ||
      m_currentViewsLayout == VPLOnly)
@@ -375,7 +366,7 @@ void QuantaView::slotSetSourceLayout()
    }
    if (m_kafkaDocument->isLoaded())
        m_kafkaDocument->unloadDocument();
-#endif
+
 //show the document if full size
    m_splitter->hide();
    m_document->view()->reparent(m_documentArea, 0, QPoint(), true);
@@ -385,17 +376,14 @@ void QuantaView::slotSetSourceLayout()
 
    m_currentViewsLayout = SourceOnly;
 
-#ifdef BUILD_KAFKAPART
 //update timers are not needed in source only mode
    m_sourceUpdateTimer.stop();
    m_VPLUpdateTimer.stop();
-#endif
 }
 
 
 void QuantaView::slotSetSourceAndVPLLayout()
 {
-#ifdef BUILD_KAFKAPART
    emit hidePreview();
    if (m_currentViewsLayout == SourceAndVPL  || !m_document)
      return;
@@ -438,12 +426,10 @@ void QuantaView::slotSetSourceAndVPLLayout()
    m_currentViewsLayout = SourceAndVPL;
 
    reloadUpdateTimers();
-#endif
 }
 
 void QuantaView::slotSetVPLOnlyLayout()
 {
-#ifdef BUILD_KAFKAPART
    emit hidePreview();
    if (m_currentViewsLayout == VPLOnly || !m_document)
      return;
@@ -487,12 +473,10 @@ void QuantaView::slotSetVPLOnlyLayout()
     m_sourceUpdateTimer.stop();
     m_VPLUpdateTimer.stop();
 
-#endif
 }
 
 void QuantaView::reloadUpdateTimers()
 {
-#ifdef BUILD_KAFKAPART
     QuantaView* view=ViewManager::ref()->activeView();
 
     m_sourceUpdateTimer.stop();
@@ -505,14 +489,12 @@ void QuantaView::reloadUpdateTimers()
        if (m_currentFocus == SourceFocus && !qConfig.kafkaRefreshOnFocus)
           m_VPLUpdateTimer.start(qConfig.kafkaRefreshDelay);
    }
-#endif
 }
 
 void QuantaView::slotVPLGetFocus(bool focus)
 {
   // is Quanta exiting?
   if (!quantaApp) return;
-#ifdef BUILD_KAFKAPART
 #ifdef LIGHT_DEBUG
   kdDebug(25001)<< "slotVPLGetFocus(" << focus << ")" << endl;
 #endif
@@ -570,18 +552,18 @@ void QuantaView::slotVPLGetFocus(bool focus)
     if(action)
       action->setEnabled(false);
 
+   //TEMPORARY: Enable VPL undo/redo logging
+   m_document->docUndoRedo->turnOn(true);
 
     m_currentFocus = VPLFocus;
     reloadUpdateTimers();
  }
-#endif
 }
 
 void QuantaView::slotSourceGetFocus()
 {
   // is Quanta exiting?
   if (!quantaApp) return;
-#ifdef BUILD_KAFKAPART
 #ifdef LIGHT_DEBUG
   kdDebug(25001)<< "slotSourceGetFocus(true)" << endl;
 #endif
@@ -634,10 +616,10 @@ void QuantaView::slotSourceGetFocus()
     if(action)
       action->setEnabled(true);
 
-#endif
-}
+    //TEMPORARY: Disable VPL undo/redo logging
+    m_document->docUndoRedo->turnOn(false);
 
-#ifdef BUILD_KAFKAPART
+}
 
 /** Reloads both views ONLY when changes have been made to the Node tree ONLY. */
 void QuantaView::reloadBothViews(bool force)
@@ -660,22 +642,17 @@ void QuantaView::reloadSourceView(bool force)
       m_document->docUndoRedo->reloadQuantaEditor(force);
 }
 
-#endif
 
 void QuantaView::VPLUpdateTimerTimeout()
 {
-#ifdef BUILD_KAFKAPART
   if(quantaApp && m_currentFocus == SourceFocus)
     reloadVPLView();
-#endif
 }
 
 void QuantaView::sourceUpdateTimerTimeout()
 {
-#ifdef BUILD_KAFKAPART
   if(quantaApp && m_currentFocus == VPLFocus)
     reloadSourceView();
-#endif
 }
 
 void QuantaView::slotVPLLoadingError(Node *)
@@ -686,12 +663,10 @@ void QuantaView::slotVPLLoadingError(Node *)
 
 void QuantaView::slotSetCursorPositionInSource(int col, int line)
 {
-#ifdef BUILD_KAFKAPART
   m_curCol = col;
   m_curLine = line;
   if (m_currentViewsLayout == SourceAndVPL || m_currentViewsLayout == SourceOnly)
     m_document->viewCursorIf->setCursorPositionReal(line, col);
-#endif
 }
 
 void QuantaView::dragEnterEvent(QDragEnterEvent *e)
@@ -726,7 +701,6 @@ void QuantaView::resize(int width, int height)
       return;
   if (m_currentViewsLayout == SourceOnly)
     m_document->view()->resize(width, height);
-#ifdef BUILD_KAFKAPART
   else
   if (m_currentViewsLayout == VPLOnly)
     m_kafkaDocument->getKafkaWidget()->view()->resize(width,height);
@@ -736,7 +710,6 @@ void QuantaView::resize(int width, int height)
     m_splitter->resize(width, height);
     m_splitterSizes = m_splitter->sizes();
   }
-#endif
 }
 
 void QuantaView::insertTag(const char *tag)
@@ -775,7 +748,10 @@ void QuantaView::insertNewTag(const QString &tag, const QString &attr, bool inse
 {
   if (m_document)
   {
-    TagDialog *dlg = new TagDialog(QuantaCommon::tagFromDTD(m_document->getDTDIdentifier(), tag), attr, baseURL());
+    QString selection;
+    if (m_document->selectionIf)  
+      selection = m_document->selectionIf->selection();
+    TagDialog *dlg = new TagDialog(QuantaCommon::tagFromDTD(m_document->getDTDIdentifier(), tag), selection, attr, baseURL());
     if (dlg->exec())
     {
       dlg->insertTag(m_document, insertInLine);
@@ -785,6 +761,7 @@ void QuantaView::insertNewTag(const QString &tag, const QString &attr, bool inse
   }
 }
 
+
 /** Returns the baseURL of the document. */
 KURL QuantaView::baseURL()
 {
@@ -808,7 +785,13 @@ void QuantaView::refreshWindow()
        resize(width(), height());
   } else
   {
-      if (m_documentArea->height() + ToolbarTabWidget::ref()->height() > height() && !ToolbarTabWidget::ref()->isHidden()) //don't use isVisible instead of isHidden!
+/*    
+    kdDebug(24000) << "m_documentArea->height(): " << m_documentArea->height() << endl;
+    kdDebug(24000) << "ToolbarTabWidget::ref()->height(): " << ToolbarTabWidget::ref()->height() << " hidden: " << ToolbarTabWidget::ref()->isHidden() << " visible: " << ToolbarTabWidget::ref()->isVisible() << endl;
+    kdDebug(24000) <<"sum: " << m_documentArea->height() + ToolbarTabWidget::ref()->height() << endl;
+    kdDebug(24000) << "height(): " << height() << endl;
+    */
+    if (m_documentArea->height() + ToolbarTabWidget::ref()->height() - 1 > height() && !ToolbarTabWidget::ref()->isHidden()) //don't use isVisible alone instead of isHidden!
         resize(m_documentArea->width(), m_documentArea->height() - ToolbarTabWidget::ref()->height());
       else if (ToolbarTabWidget::ref()->isHidden())
         resize(width(), height());
@@ -829,7 +812,17 @@ void QuantaView::activated()
   quantaApp->partManager()->setActivePart(m_document->doc(), m_document->view());
   m_document->checkDirtyStatus();
   StructTreeView::ref()->useOpenLevelSetting = true;
-  quantaApp->loadToolbarForDTD(m_document->getDTDIdentifier());
+  quantaApp->slotLoadToolbarForDTD(m_document->getDTDIdentifier());
+
+  //TEMP : If the activated document is not a (X)HTML document, disable smartTagInsertion
+  //Will be removed when VPL will support every DTD
+  KAction *action = quantaApp->actionCollection()->action("smart_tag_insertion");
+  if(action && m_document->defaultDTD()->name.contains("HTML", false) == 0)
+  {
+    qConfig.smartTagInsertion = false;
+    (static_cast<KToggleAction* >(action))->setChecked(false);
+  }
+
   switch (m_currentViewsLayout)
   {
       case SourceOnly:
@@ -861,13 +854,11 @@ void QuantaView::deactivated()
   {
     quantaApp->statusBar()->changeItem("", IDS_STATUS);
   }
-#ifdef BUILD_KAFKAPART
   m_sourceUpdateTimer.stop();
   m_VPLUpdateTimer.stop();
-#endif
 }
 
-bool QuantaView::saveModified()
+bool QuantaView::saveModified(bool ask)
 {
   if (!m_document)
     return true;
@@ -877,17 +868,22 @@ bool QuantaView::saveModified()
 
   if (m_document->isModified() )
   {
-    int want_save
-      = KMessageBox::warningYesNoCancel(this,
+    if (m_currentFocus == VPLFocus)
+      reloadSourceView();
+    int want_save;
+    if (ask)
+      want_save = KMessageBox::warningYesNoCancel(this,
           i18n("The file \"%1\" has been modified.\nDo you want to save it?").arg(fileName),
           i18n("Warning"), KStdGuiItem::save(), KStdGuiItem::discard());
+    else
+      want_save = KMessageBox::Yes;
 
     switch (want_save)
     {
       case KMessageBox::Yes :
            if (m_document->isUntitled())
            {
-             completed = quantaApp->slotFileSaveAs();
+             completed = quantaApp->slotFileSaveAs(this);
            }
            else
            {
@@ -898,7 +894,7 @@ bool QuantaView::saveModified()
 
       case KMessageBox::No :
            {
-	     m_document->removeBackup(quantaApp->config());
+      	     m_document->removeBackup(quantaApp->config());
       	     completed=true;
 	   }
            break;
@@ -911,7 +907,8 @@ bool QuantaView::saveModified()
            completed=false;
            break;
     }
-  }
+  } else
+    m_document->removeBackup(quantaApp->config());
   return completed;
 }
 
@@ -926,23 +923,21 @@ bool QuantaView::saveDocument(const KURL& url)
   if (!m_document->isUntitled() && oldURL.isLocalFile())
   {
     fileWatcher->removeFile(oldURL.path());
-    kdDebug(24000) << "removeFile[saveDocument]: " << oldURL.path() << endl;
+//    kdDebug(24000) << "removeFile[saveDocument]: " << oldURL.path() << endl;
   }
   if (url.isLocalFile())
   {
     if (!m_document->doc()->saveAs(url))
     {
       fileWatcher->addFile(oldURL.path());
-      kdDebug(24000) << "addFile[saveDocument]: " << oldURL.path() << endl;
+//      kdDebug(24000) << "addFile[saveDocument]: " << oldURL.path() << endl;
       return false; //saving to a local file failed
     } else //successful saving to a local file
     {
-      m_document->closeTempFile();
-      m_document->createTempFile();
       m_document->setDirtyStatus(false);
       m_document->removeBackup(quantaApp->config());
       fileWatcher->addFile(m_document->url().path());
-      kdDebug(24000) << "addFile[saveDocument, 2]: " << m_document->url().path() << endl;
+//      kdDebug(24000) << "addFile[saveDocument, 2]: " << m_document->url().path() << endl;
     }
   } else //saving to a remote file
   {
@@ -965,17 +960,17 @@ bool QuantaView::saveDocument(const KURL& url)
       if (oldURL.isLocalFile())
       {
         fileWatcher->addFile(oldURL.path());
-        kdDebug(24000) << "addFile[saveDocument, 3]: " << oldURL.path() << endl;
+//        kdDebug(24000) << "addFile[saveDocument, 3]: " << oldURL.path() << endl;
       }
       return false;
     }
-    #if KDE_VERSION < KDE_MAKE_VERSION(3,2,90) // TODO: remove if support for 3.2 is dropped
+#if KDE_VERSION < KDE_MAKE_VERSION(3,2,90) // TODO: remove if support for 3.2 is dropped
     // notify the treeviews
     KDirNotify_stub allDirNotify("*", "KDirNotify*");
     KURL dirUrl( url );
     dirUrl.setPath( dirUrl.directory() );
     allDirNotify.FilesAdded( dirUrl );
-    #endif
+#endif
   }
   // everything went fine
   if (oldURL != m_document->url())
@@ -997,8 +992,6 @@ void QuantaView::slotSavingFailed(const QString &error)
 void QuantaView::slotSavingCompleted()
 {
   m_saveResult = true;
-  m_document->closeTempFile();
-  m_document->createTempFile();
   m_document->setDirtyStatus(false);
   m_document->removeBackup(quantaApp->config());
   if (m_eventLoopStarted)
