@@ -22,15 +22,17 @@
 #include <qtooltip.h>
 
 //kde includes
-//#include <klistview.h>
+#include <kaction.h>
 #include <klocale.h>
 #include <kpushbutton.h>
 #include <kstringhandler.h>
 #include <kdebug.h>
 #include <kiconloader.h>
+#include <khtmlview.h>
 #include <ktexteditor/editinterface.h>
 #include <ktexteditor/viewcursorinterface.h>
 #include <ktexteditor/view.h>
+#include <dom/dom_node.h>
 
 //app includes
 #include "tagattributetree.h"
@@ -42,15 +44,9 @@
 #include "document.h"
 #include "quantaview.h"
 #include "tag.h"
-#include "resource.h"
-#include "quanta.h"
-#ifdef BUILD_KAFKAPART
-#include <dom/dom_node.h>
-#include <kaction.h>
 #include "wkafkapart.h"
 #include "kafkacommon.h"
 #include "undoredo.h"
-#endif
 
 #include "viewmanager.h"
 
@@ -256,11 +252,9 @@ void TagAttributeTree::setCurrentNode(Node *node)
     m_node = node = node->parent;
   if (!node)
       return;
-#ifdef BUILD_KAFKAPART
 #ifdef HEAVY_DEBUG
   kafkaCommon::coutTree(baseNode, 2);
   KafkaDocument::ref()->coutLinkTree(baseNode, 2);
-#endif
 #endif
   AttributeItem *item = 0L;
   TopLevelItem *group = 0L;
@@ -386,107 +380,24 @@ void TagAttributeTree::editorContentChanged()
     if (dynamic_cast<AttributeNameSpaceItem*>(item))
     {
       QString nameSpace = item->editorText();
-      if (!nameSpace.isEmpty() && m_node->tag->type == Tag::XmlTagEnd)
-        nameSpace.prepend('/');
       m_node->tag->write()->changeTagNamespace(m_node->tag, nameSpace);
     } else
     {
-#ifdef BUILD_KAFKAPART
     if(ViewManager::ref()->activeView()->hadLastFocus() == QuantaView::SourceFocus)
     {
-#endif
       m_node->tag->write()->changeTagAttribute(m_node->tag, item->text(0), item->editorText());
-#ifdef BUILD_KAFKAPART
     }
     else
     {
-      //edit only the tag and its corresponding DOM::Node
-      int i;
-      TagAttr attr;
-      DOM::Node domNode;
-      bool foundAttr = false, nodeModified= false;
+      //edit the attribute
       NodeModifsSet *modifs = new NodeModifsSet();
-      NodeModif *modif = new NodeModif();
-      modif->setType(NodeModif::NodeModified);
-      modif->setTag(new Tag(*(m_node->tag)));
-      modif->setLocation(kafkaCommon::getLocation(m_node));
+      kafkaCommon::editNodeAttribute(m_node, item->text(0), item->editorText(), modifs);
+      ViewManager::ref()->activeDocument()->docUndoRedo->addNewModifsSet(modifs, undoRedo::NodeTreeModif);
 
-      for(i = 0; i < m_node->tag->attrCount(); i++)
-      {
-        if(m_node->tag->attribute(i) == item->text(0))
-        {
-          foundAttr = true;
-#ifdef HEAVY_DEBUG
-          kdDebug(25001)<< "Attribute " << item->text(0) << " found!" << endl;
-#endif
-          if(m_node->tag->attributeValue(i) != item->editorText())
-          {
-#ifdef HEAVY_DEBUG
-          kdDebug(25001)<< "Different attribute value " << item->editorText() << ", deleting the attr." << endl;
-#endif
-            //delete the attribute in the Node
-            attr = m_node->tag->getAttribute((unsigned)i);
-            m_node->tag->deleteAttribute((unsigned)i);
-
-            m_node->tag->cleanStrBuilt = false;
-            nodeModified = true;
-          }
-          break;
-        }
-      }
-      if((!foundAttr && item->editorText() != "") ||
-        (foundAttr && item->editorText() != m_node->tag->attributeValue(i) && item->editorText() != ""))
-      {
-#ifdef HEAVY_DEBUG
-         kdDebug(25001)<< "(re)creating the attr." << endl;
-#endif
-         //(re)create the attr.
-         attr.name = item->text(0);
-         attr.value = item->editorText();
-         attr.quoted = true;
-         m_node->tag->addAttribute(attr);
-
-         m_node->tag->cleanStrBuilt = false;
-         nodeModified = true;
-      }
-      QuantaView *view = ViewManager::ref()->activeView();
-      if(nodeModified)
-      {
-        //delete the corresponding DOM::Node.
-        if(m_node->rootNode())
-        {
-          domNode = *m_node->rootNode();
-          KafkaDocument::ref()->disconnectDomNodeFromQuantaNode(domNode);
-          domNode.parentNode().removeChild(domNode);
-        }
-        if(m_node->leafNode() && m_node->rootNode() && *m_node->rootNode() != *m_node->leafNode())
-        {
-          domNode = *m_node->leafNode();
-          KafkaDocument::ref()->disconnectDomNodeFromQuantaNode(domNode);
-          domNode.parentNode().removeChild(domNode);
-        }
-        if(m_node->rootNode())
-          delete m_node->rootNode();
-        m_node->setRootNode(0L);
-        if(m_node->leafNode())
-          delete m_node->leafNode();
-        m_node->setLeafNode(0L);
-
-        KafkaDocument::ref()->buildKafkaNodeFromNode(m_node);
-        if(!domNode.isNull() && m_node->leafNode())
-        {
-           while(!domNode.firstChild().isNull())
-             m_node->leafNode()->appendChild(domNode.firstChild());
-        }
-
-        modifs->addNodeModif(modif);
-        view->document()->docUndoRedo->addNewModifsSet(modifs, undoRedo::NodeTreeModif);
-      }
 #ifdef HEAVY_DEBUG
         kafkaCommon::coutTree(baseNode, 2);
 #endif
       }
-#endif
     }
     rebuildEnabled = true;
   }
@@ -532,7 +443,12 @@ void TagAttributeTree::slotDelayedSetCurrentNode()
 {
   setCurrentNode(m_newNode);
   if (ViewManager::ref()->activeDocument())
-    ViewManager::ref()->activeDocument()->view()->setFocus();
+  {
+    if (ViewManager::ref()->activeView()->hadLastFocus() == QuantaView::SourceFocus)
+      ViewManager::ref()->activeDocument()->view()->setFocus();
+    else
+      KafkaDocument::ref()->getKafkaWidget()->view()->setFocus();
+  }
 }
 
 EnhancedTagAttributeTree::EnhancedTagAttributeTree(QWidget *parent, const char *name)
@@ -616,7 +532,6 @@ void EnhancedTagAttributeTree::deleteSubTree()
   QuantaView *view = ViewManager::ref()->activeView();
   if(!curNode || !view->document())
     return;
-#ifdef BUILD_KAFKAPART
   Node *oldCurNode;
   NodeModifsSet *modifs;
   int curLine, curCol, offset;
@@ -661,32 +576,6 @@ void EnhancedTagAttributeTree::deleteSubTree()
     KafkaDocument::ref()->getKafkaWidget()->document());
     KafkaDocument::ref()->getKafkaWidget()->setCurrentNode(domNode, offset);
   }
-
-#else
-
-  //Quick code for the 3.1.x users
-
-  Node *oldNode;
-  int bCol, bLine, eCol, eLine;
-  oldNode = curNode;
-  curNode = 0L;
-  attrTree->setCurrentNode(curNode);
-
-  oldNode->tag->beginPos(bLine, bCol);
-  if(oldNode->tag->type == Tag::XmlTag && !oldNode->tag->single)
-  {
-    if(oldNode->getClosingNode())
-      oldNode->getClosingNode()->tag->endPos(eLine, eCol);
-    else if(oldNode->child)
-      oldNode->lastChild()->tag->endPos(eLine, eCol);
-    else
-      oldNode->tag->endPos(eLine, eCol);
-  }
-  else
-    oldNode->tag->endPos(eLine, eCol);
-
-  view->document()->editIf->removeText(bLine, bCol, eLine, eCol+1);
-#endif
 }
 
 void EnhancedTagAttributeTree::deleteNode()
@@ -695,7 +584,6 @@ void EnhancedTagAttributeTree::deleteNode()
   if(!curNode || !view->document())
     return;
 
-#ifdef BUILD_KAFKAPART
   Node *oldCurNode, *oldCurNodeParent, *child;
   QTag *oldCurNodeParentQTag;
   int curLine, curCol, offset;
@@ -764,28 +652,6 @@ void EnhancedTagAttributeTree::deleteNode()
     KafkaDocument::ref()->getKafkaWidget()->setCurrentNode(domNode, offset);
   }
 
-#else
-  //Quick code for 3.1.x users
-  //Without kafka, all is more text-based, so we can avoid removing Nodes with wrong DTD relationship.
-
-  Node *oldNode;
-  int bCol, bLine, eCol, eLine;
-  oldNode = curNode;
-  curNode = 0L;
-  attrTree->setCurrentNode(curNode);
-
-  if(oldNode->tag->type == Tag::XmlTag && !oldNode->tag->single && oldNode->getClosingNode())
-  {
-    oldNode->getClosingNode()->tag->beginPos(bLine, bCol);
-    oldNode->getClosingNode()->tag->endPos(eLine, eCol);
-    view->document()->editIf->removeText(bLine, bCol, eLine, eCol+1);
-  }
-
-  oldNode->tag->beginPos(bLine, bCol);
-  oldNode->tag->endPos(eLine, eCol);
-  view->document()->editIf->removeText(bLine, bCol, eLine, eCol+1);
-
-#endif
 }
 
 #include "tagattributetree.moc"
